#!/usr/bin/env python3
# *****************************************************************************
"""
                                   Merge Images
 
         Overview: This program imports various Python modules and calls their
                   main methods to utilize their various features utilizing a
                   standard interface. 
 
        Author(s): Josh Sackos
         Revision: 0.0.1
    Revision Date: 12/07/2025
    Tool Versions: Python     3.13.3
                   Numpy      2.2.6
 
            Notes: None
"""
# *****************************************************************************

# ----* Native Imports *----
import sys;
import os;
import importlib;
from   importlib import resources;

# ----* Third Party Imports *----
import numpy as np;
from SackosArt.types.SackosScriptResult import SackosScriptResult;

def get_img_path(img:str) -> str:
    base_path = os.path.dirname(__file__);
    return os.path.join(base_path, 'assets', img);

def main(*args, **kwargs) -> int:
    # //////////////////////////////////////////////////////////////////////
    """"
           Description: Main program function/application entry point.
       
             Arguments:
                         args   : list : cli arguments (none expected)
                         kwargs : dict : cli arguments (none expected)

               Returns:  int : 0 if success, else error
    """
    # //////////////////////////////////////////////////////////////////////

    # --* Script Setup *--
    ret_val = SackosScriptResult(); # For storing return data
    # img1    = str(resources.files("SackosArt.examples.data").joinpath("img1.png"));
    # img2    = str(resources.files("SackosArt.examples.data").joinpath("img2.png"));
    img1 = "img1.png";
    img2 = "img2.png";

    # ---- Import Module to Merge Two Images ----
    module  = "SackosArt.examples.merge_two_images";
    tmp_lib = importlib.import_module(module);

    # ---- Merge Two Images ----
    module_args = {
                   'image1'           : img1,
                   'image1_intensity' : 50  ,
                   'image2'           : img2,
                   'image2_intensity' : 50  ,
                   'imported'         : True
                  };
    ret_val = tmp_lib.main(**module_args);
    myvar = sys.modules;
    del sys.modules[module];    # Unload the Module

    # ---- Spiral-Merge Image2 into Image1 ----
    # NOT READ YET BUT SHOULD BE SOON ;)
    return ret_val["exit_status"];   # Return success?

# -----------------------------------------------------------------------------
#                           Application Entry Point
# -----------------------------------------------------------------------------
if __name__ == "__main__":
    raise SystemExit(main(sys.argv));



#!/usr/bin/env python3
# *****************************************************************************
"""                           SackosResult Class
 
         Overview: The SackosResult is a class used for returning complex
                   data from a Sackos-Art script main() methods. The need
                   for this arises out of the fact that a script's main()
                   method may either be called from the command-line or
                   directly when the script is imported.
 
        Author(s): Josh Sackos
         Revision: 0.0.1
    Revision Date: 12/28/2025
    Tool Versions: Python     3.13.3
                   Numpy      2.2.6
 
            Notes: None
"""
# *****************************************************************************

# ----* Native Imports *----
import sys, os;
from   typing import TypedDict, Dict;

# ----* Third Party Imports *----
import numpy as np;
from SackosArt.types import CoordSys

class __SackosScriptResult(Dict):
    # ======================================================================
    """Description: Custom dictionary for Sackos-Art script return values.
                    Given the complex data structures arbitrarily returned
                    by the scripts, this class acts as a means to provide 
                    an IDE/Intellisense friendly interace to the data in  
                    addition to providing a flexible/manageable way to
                    accomplish the desired functionality.

        Attributes: version : Current version of class.

          Examples: None
           Related: None
    """
    # ======================================================================

    # ++++ Required Dictionary Key/Val Pairs ++++ 
    DEFAULTS = {
                'exit_status': 0,
                'val1': 1,
                'val2': 2,
                'val3': 3,
                'val4': 4                

    };

    # ++++ Class Variables ++++
    version = 0.1;


    def __init__(self, *args, **kwargs) -> None:
        # ///////////////////////////////////////////////////////////////
        """Description: Class constructor
             Arguments: None
               Returns: None
        """
        # ///////////////////////////////////////////////////////////////
        super().__init__(*args, **kwargs);

        # --* Create Default Key/Val Pairs Here *--
        for key, val in self.DEFAULTS.items():
          self.setdefault(key, val);
        return;

    def __del__(self) -> None:
        # ///////////////////////////////////////////////////////////////
        """Description: Class destructor
             Arguments: None
               Returns: None
        """
        # ///////////////////////////////////////////////////////////////

        return;

class SackosScriptResult(__SackosScriptResult):
    # ======================================================================
    """Description: Class annotation layer for __SackosScriptResult class.
                    Allows for IDE/Intellisense friendly custom dictionary
                    return values used in SackosArt scripts.
    """
    # ======================================================================

    # --* Annotations for IDE/Intellisense *--
    exit_status: int;

# -----------------------------------------------------------------------------
#                           Application Entry Point
# -----------------------------------------------------------------------------
if __name__ == "__main__":

    mydict = SackosScriptResult();
    sa.pprint(mydict);

    raise SystemExit(0);



#!/usr/bin/env python3
# *****************************************************************************
"""
                                   Merge Images
 
         Overview: This program imports 2 JPEG images using OpenCV-4, extracts
                   the raw pixel data, merges the two images together with
                   variable pixel intensities for each respective image.
 
        Author(s): Josh Sackos
         Revision: 0.0.1
    Revision Date: 12/07/2025
    Tool Versions: Python     3.13.3
                   OpenCV 4   4.11.0
                   Numpy      2.2.6
 
            Notes: Still need to account for pixel value overflow.
"""
# *****************************************************************************

# ----* Native Imports *----
import sys;
import os;
from   importlib import resources;
from   pathlib   import Path;

# ----* Third Party Imports *----
import numpy as np;
import cv2;
from SackosArt.types.SackosScriptResult import SackosScriptResult;


def main(*args, **kwargs) -> SackosScriptResult:
    # //////////////////////////////////////////////////////////////////////
    """
           Description: Main program function/application entry point.
             Arguments:
                         args : list : Command-line arguments
       
                                       args[0] = image1 path
                                       args[1] = image1 pixel intensity
                                       args[2] = image2 path
                                       args[3] = image2 pixe2 intensity

                         kwargs : dict : Same as Command-line arguments
                                         but in dictionary form. Key/Value
                                         pairs will be interpreted left to
                                         right and only values extracted.

                                         NOTE: kwargs not used if args
                                               is used and vice versa.
       
                         Example Arguments:
                                            args[0] = georgy_feet.jpg
                                            args[1] = 50
                                            args[2] = georgy_flower.jpg
                                            args[3] = 75

               Returns:  dict : {'exit_status':int}  : System Exit Status
                                {'result':np.ndarray}: Merged OpenCV 4 Img

                 Notes: Returns a dict containing the result/merged image
                        numpy array. (Allows for uniform interface between
                        unrelated Python modules)
    """
    # //////////////////////////////////////////////////////////////////////

    # --* Script Setup *--
    ret_val = SackosScriptResult(); # For all SackosArt script return values

    # ---- Process CLI Arguments ----
    isCLI          = False if "imported" in kwargs.keys() else True;
    args           = args[0] if isCLI else [];
    # Convert dictionary kwargs to list style since it was already written :P
    for i, (key, val) in enumerate(kwargs.items()):
        if i >= 4:
            break;
        args.append(val);

    # Extract image parameters
    file1      = resources.files("SackosArt.examples.data").joinpath(args[0]);
    intensity1 = (((args[1] % 100)/100.0) if args[1] != 100 else 1.0) \
                    if ( args[1] > 1.0 ) else ( args[1] );
#    fname1         = os.path.abspath(file1).replace('\\', '/').split('/')[-1];

    file2       = resources.files("SackosArt.examples.data").joinpath(args[2]);
    intensity2  = (((args[3] % 100)/100.0) if args[3] != 100 else 1.0) \
                     if ( args[3] > 1.0 ) else ( args[3] );
#    fname2         = os.path.abspath(file2).replace('\\', '/').split('/')[-1];
    
    # ----- Import Images -----
    try:
        img1  = cv2.imread(str(file1), cv2.IMREAD_COLOR);
        img2  = cv2.imread(str(file2), cv2.IMREAD_COLOR);
    except:
        print("ERROR: Could not open images, aborting...");
        return {'exit_status':-1};

    # ----- Resize Images -----
    img1_scaled = cv2.resize(img1, None, fx=0.25, fy=0.25, \
                             interpolation=cv2.INTER_AREA);
    img2_scaled = cv2.resize(img2, None, fx=0.25, fy=0.25, \
                             interpolation=cv2.INTER_AREA);

    # ----- Create Destination Matrix -----
    img = np.zeros(img1_scaled.shape, dtype=np.uint8);

    # ----- Merge Both Images -----
    img[:, :, 0] = (img1_scaled[:,:,0]*intensity1) + \
                   (img2_scaled[:,:,0]*intensity2);
    img[:, :, 1] = (img1_scaled[:,:,1]*intensity1) + \
                   (img2_scaled[:,:,1]*intensity2);
    img[:, :, 2] = (img1_scaled[:,:,2]*intensity1) + \
                   (img2_scaled[:,:,2]*intensity2);
    ret_val['result'] = img;

    # ----- Show Merged Image -----
    showImage('Merged Image' , img);
    
    # ---- Save Image ----
    fname1   = file1.name.split('.');
    fname2   = file2.name.split('.');
    save_ext = fname1[1:2][0];

    save_path = str(Path.cwd())+'/'+fname1[0]+'_'+fname2[0]+'.'+save_ext;

    # save_path = '/'.join(os.path.abspath(file1).replace('\\', '/').split("/")\
    #             [0:-1]) + '/' + fname1 + "_" + fname2 + '.jpg';
    
    cv2.imwrite(save_path, img);

    # ---- Wait for Keypress ----
    cv2.waitKey(0);

    # ---- Return Success & Result ----
    return ret_val;

def showImage(wTitle, img) -> None:
    # //////////////////////////////////////////////////////////////////////
    """
            Description: Displays numpy matrix as an image via OpenCV
                         imshow().
    
              Arguments:
                         wTitle : str     : Window title
                         img    : ndarray : Numpy matrix
    
                Returns: None
    """
    # //////////////////////////////////////////////////////////////////////
    cv2.imshow(wTitle, img);
    return;

# -----------------------------------------------------------------------------
#                           Application Entry Point
# -----------------------------------------------------------------------------
if __name__ == "__main__":

    try:
        args    = sys.argv[1:5];
        args[0] = args[0].replace('\\', '/'); # Use unix filepath separator
        args[1] = float(args[1]);             # Convert intensity to integer
        args[2] = args[2].replace('\\', '/'); # Use unix filepath separator
        args[3] = float(args[3]);             # Convert intensity to integer

    except:
        print("Invalid arguments. Expected Command Line Arguments:");
        print("\targs[0] = image1 path"                            );
        print("\targs[1] = image1 pixel intensity"                 );
        print("\targs[2] = image2 path"                            );
        print("\targs[3] = image2 pixel intensity"                 );
        raise SystemExit(-1); # Return error

#    raise SystemExit(main(args, imported=True)[0]);
    raise SystemExit(main(args)['exit_status']);



#!/usr/bin/env python3
# *****************************************************************************
"""
                                   Merge Images
 
         Overview: This program imports 2 JPEG images using OpenCV-4, extracts
                   the raw pixel data, merges the two images together with
                   variable pixel intensities for each respective image.
 
        Author(s): Josh Sackos
         Revision: 0.0.1
    Revision Date: 12/07/2025
    Tool Versions: Python     3.13.3
                   OpenCV 4   4.11.0
                   Numpy      2.2.6
 
            Notes: Definitely not working yet!
"""
# *****************************************************************************

# ----* Native Imports *----
import sys;
from   typing import Any;
import os;

# ----* Third Party Imports *----
import numpy as np;
import cv2;
from SackosArt.types.SackosScriptResult import SackosScriptResult;

def main(
         *args     : tuple[Any , ...],
         **kwargs  : dict[Any  , Any]
    ) -> SackosScriptResult[Any,Any]:
    # //////////////////////////////////////////////////////////////////////
    """Description: Main program function/application entry point.
        Arguments:

          Returns:  dict : {'exit_status':int}  : System Exit Status
                           {'result':np.ndarray}: Merged OpenCV 4 Img

          Notes: Returns a dict containing the result/merged image
                 numpy array. (Allows for uniform interface between
                 unrelated Python modules)
    """
    # //////////////////////////////////////////////////////////////////////

    # --* Script Setup *--
    ret_val = SackosScriptResult(); # For storing return data

    print("'sprial_effect' module is NOT ready yet. Please check back soon!");

    # ---- Return success command-line, and image if imported by module ----
    ret_val['exit_status'] = 0;
    return ret_val;

# -----------------------------------------------------------------------------
#                           Application Entry Point
# -----------------------------------------------------------------------------
if __name__ == "__main__":

    raise SystemExit(main()['exit_status']);



#!/usr/bin/env python3
# *****************************************************************************
"""                            CoordSys Class
 
         Overview: Implementation of the CoordSys parent class that is used
                   for defining new coordinate systems for use in the
                   Sackos-Art software.
 
        Author(s): Josh Sackos
         Revision: 0.0.1
    Revision Date: 12/25/2025
    Tool Versions: Python     3.13.3
                   Numpy      2.2.6
 
            Notes: None
"""
# *****************************************************************************

# ----* Native Imports *----
import sys;
import os;

# ----* Third Party Imports *----
import numpy as np;
import matplotlib.pyplot as plt;
import math;
import SackosArt as sa;

class CoordSys():
    # ======================================================================
    """Description: Parent class for coordinate systems used by
                    Sackos-Art software.

        Attributes: version : Current version of class.

          Examples: None
           Related: None
    """
    # ======================================================================

    # ++++ Class Variables ++++
    version = 0.1;

    def __init__(self) -> None:
        # ///////////////////////////////////////////////////////////////
        """Description: Class constructor
             Arguments: None
               Returns: None
        """
        # ///////////////////////////////////////////////////////////////
        self.pi = math.pi;

        return;

    def __del__(self) -> None:
        # ///////////////////////////////////////////////////////////////
        """Description: Class destructor
             Arguments: None
               Returns: None
        """
        # ///////////////////////////////////////////////////////////////

        return;

    def plot(self, data, system):
        # ///////////////////////////////////////////////////////////////
        """Description: 
             Arguments: 
               Returns:
        """
        # ///////////////////////////////////////////////////////////////
        system = system.lower();
        fig    = plt.figure();

        # --* Polar Plot *--
        if system == 'polar':
            r     = [x for x,y in data];
            theta = [y for x,y in data];
            plt.polar(theta, r, marker='o');
        
        # --* Cartesian Plot *--
        elif system == 'cartesian':
            x = [x1 for x1,y1 in data];
            y = [y1 for x1,y1 in data];
            plt.plot(x,y, marker='p');
        
        # --* Cylindrical Plot *--
        elif system == 'cylindrical':
          pass;

        # --* Spherical Plot *--
        elif system == 'spherical':
          pass;

        # --* Handle mouse clicks for data points *--
        def onclick(event):
            if event.xdata is not None and event.ydata is not None:
                print(f"Clicked at: ({event.xdata:.3f}, {event.ydata:.3f})")
        fig.canvas.mpl_connect('button_press_event', onclick)


        # --* Show Plot *--
        plt.show();

        return;

    def rad2deg(self,
                rads  : np.ndarray[float] | float,
                reduce: bool=True
        ) -> list[float] | float:
        # ///////////////////////////////////////////////////////////////
        """Description: Converts rad to degrees.
             Arguments: 
                        rads   : Converts radian value(s) to degrees
                        reduce : Mod degree values so no result exceeds
                                 360° 
                                       -> True  = Reduce values
                                       -> False = Preserve value 

               Returns: rad value(s) in degrees
        """
        # ///////////////////////////////////////////////////////////////
        degrees = rads*(180.0/self.pi);
        return( (degrees % 360.0) if reduce else degrees );

    def deg2rad(self,
                degs  : np.ndarray[float] | float,
                reduce: bool=True
        ) -> np.ndarray[float] | float:
        # ///////////////////////////////////////////////////////////////
        """Description: Converts degrees to rad.
             Arguments: 
                        degs   : Degree value(s) to convert.
                        reduce : Mod degree values so no
                                 result exceeds 2π.
                                     -> True  = Reduce values
                                     -> False = Preserve value 

               Returns: rad value in degrees
        """
        # ///////////////////////////////////////////////////////////////
        rad = (degs*(self.pi/180.0));
        return( (rad % (2*self.pi)) if reduce else rad );

    def cart2polar(self,
                   coords: list[tuple[float,float]]
        ) -> list[tuple[float,float]]:
        # ///////////////////////////////////////////////////////////////
        """Description: Converts Cartesian coordinates to Polar
                        coordinates.
             Arguments: 
                        coords : List of (x,y) tuples to be converted to
                        polar coordinate.

               Returns:  Polar coordinate equivalent tuples
                         in the form of (r,θ).
                         
                         NOTE: θ = rad
        """
        # ///////////////////////////////////////////////////////////////
        return [(math.sqrt((x**2+y**2)),math.atan2(y,x)) for x,y in coords];

    def polar2cart(self,
                   coords: list[tuple[float,float]]
        ) -> list[tuple[float,float]]:
        # ///////////////////////////////////////////////////////////////
        """Description: Converts Polar coordinates to Cartesian
                        coordinates.
             Arguments: 
                        coords : List of (r,θ) polar tuples to be
                                 converted to Cartesian coordinates. 
                                 
                                 NOTE: θ = radian.

               Returns: Cartesian coordinate equivalent tuples in the
                        form of (x,y).
        """
        # ///////////////////////////////////////////////////////////////
        return [(r*math.cos(θ),r*math.sin(θ)) for r,θ in coords];

# -----------------------------------------------------------------------------
#                           Application Entry Point
# -----------------------------------------------------------------------------
if __name__ == "__main__":

    reduce        = False;
    coordsys_inst = CoordSys();

    # --* Cartesian Example *--
    cart_coords = [
                    (  math.sqrt (2)/2 ,  math.sqrt(2)/2 ), # Quadrant 1 @ 45°
                    ( -math.sqrt (2)/2 ,  math.sqrt(2)/2 ), # Quadrant 2 @ 135°
                    ( -math.sqrt (2)/2 , -math.sqrt(2)/2 ), # Quadrant 3 @ 225°
                    (  math.sqrt (2)/2 , -math.sqrt(2)/2 )  # Quadrant 4 @ 315°
    ];

    polar_coords = coordsys_inst.cart2polar(cart_coords);
#    coordsys_inst.plot(polar_coords, system='polar');

    # --* Polar Example *--
    polar_coords = [
                    (  1 ,    math.pi /4.0 ), # Quadrant 1 @ 45°
                    (  1 , (3*math.pi)/4.0 ), # Quadrant 2 @ 135°
                    (  1 , (5*math.pi)/4.0 ), # Quadrant 3 @ 225°
                    (  1 , (7*math.pi)/4.0 )  # Quadrant 4 @ 315°
    ];
    cart_coords = coordsys_inst.polar2cart(polar_coords);
#    coordsys_inst.plot(cart_coords, system='cartesian');

    raise SystemExit(0);
